#include <stdio.h>
#include "LIB/message.h"
#include "LIB/constmap.h"
#include "LIB/dimension.h"
#include "LIB/menu.h"

#define NPAR 3

extern float *farray1();
extern float Legpolyn();
extern float cub_bspline();
extern float spl();
extern void free_farray1();
extern FILE *getfp(char *);
void harmogrid(int maxs,float *coef,char *conven,char ifset);

void grids3d(void){
char disc[4][6],string[40],ifset;
float *coef;
FILE *fp;
int maxs,kmaxw,kmaxu,kmaxl,ndis,kmax,jump,hmax,mdim,vjump,i,p;
float mindep,maxdep,fac,x,r,*mdl;

  fp=getfp("r");
  fscanf(fp,"%d %d %d %d %d",&maxs,&kmaxw,&kmaxu,&kmaxl,&ndis);
  if(maxs>MAXS) STOP(grids3d: maxs too large);
  
  if(kmaxw>=0 || (kmaxu>=0 && kmaxl>=0) )
  { mindep=RADIUS-RMOHO;
    maxdep=RADIUS-RCMB;
  }
  else if(kmaxu>=0)
  { mindep=RADIUS-RMOHO;
    maxdep=RADIUS-R670;
  }
  else if(kmaxl>=0)
  { mindep=RADIUS-R670;
    maxdep=RADIUS-RCMB;
  }
  else 
  { mindep=0.0;
    maxdep=0.0;
  }
  printf(" Depth range: [%4d %4d] km", (int)mindep,(int)maxdep);
  
  disc[0][0]=disc[1][0]=disc[2][0]=disc[3][0]='\0'; 
  if(ndis)
  { printf(";  Discontinuities:");
    for(i=0;i<ndis;i++)
    { fscanf(fp,"%s",disc[i]);
      printf(" %s",disc[i]);
    }
  }
  printf(" or 'q' to quit:");
  printf("\n");
  
  hmax=(maxs+1)*(maxs+1);
  mdim=hmax*(ndis+kmaxw+1+kmaxu+1+kmaxl+1);
  mdl=farray1(0,mdim-1);
  for(i=0;i<mdim;i++) fscanf(fp,"%g",&mdl[i]);
  fclose(fp);
  
  vjump=hmax*(kmaxw+1+kmaxu+1+kmaxl+1);
  
  coef=farray1(0,hmax-1);
  
  ifset='y';
  do {
    printf(" Specify a depth,");
    if(ndis)
    { printf(" or ");
      for(i=0;i<ndis;i++) printf("'%c' ",disc[i][0]);
      printf(" for discontinuity,");
    }
    scanf("%s",string);
    if(string[0]=='q')break;
    
    if(string[0]=='s') for(i=0;i<hmax;i++) coef[i]=mdl[vjump+i]*6368.0;
    else if(string[0]=='m')
    { if(disc[0][0]=='m') jump=0;
      else if(disc[1][0]=='m') jump=hmax;
      else STOP(grids3d: error in getting moho);
      for(i=0;i<hmax;i++) coef[i]=mdl[vjump+jump+i]*RMOHO;
    }
    else if(string[0]=='d')
    { if(disc[0][0]=='d') jump=0;
      else if(disc[1][0]=='d') jump=hmax;
      else if(disc[2][0]=='d') jump=2*hmax;
      else STOP(grids3d: error in getting d670);
      for(i=0;i<hmax;i++) coef[i]=mdl[vjump+jump+i]*R670;
    }
    else if(string[0]=='c')
      for(i=0;i<hmax;i++) coef[i]=mdl[vjump+hmax*(ndis-1)+i]*RCMB;
    else if(string[0]>='1' && string[0]<='9')
    { sscanf(string,"%f",&r);
      r=RADIUS-r;
      
      if(kmaxw<0)
      { if(r>R670)
        { if(kmaxu<0) STOP(grids3d: upper mantle not available);
	  x=-1.+2.*(r-R670)/(RMOHO-R670);
	  kmax=kmaxu;
	  jump=0;
        }
        else
        { if(kmaxl<0) STOP(grids3d: lower mantle not available);
	  x=-1.+2.*(r-RCMB)/(R670-RCMB);
	  kmax=kmaxl;
	  jump=(kmaxu+1)*hmax;
        }
      }
      else
      { if(kmaxu>=0 || kmaxl>=0) STOP(grids3d: error in s3d file);
	x=-1.+2.*(r-RCMB)/(RMOHO-RCMB);
	kmax=kmaxw;
	jump=0;
      }
      
      for(i=0;i<hmax;i++) coef[i]=0;
      for(p=0;p<=kmax;p++)
      { fac=100.*Legpolyn(p,x);
        for(i=0;i<hmax;i++) coef[i]+=fac*mdl[jump+i+p*hmax];
      }
    }
    if(string[0]!='q')string[0]='\0';
    harmogrid(maxs,coef,"X^",ifset);

    ifset='n';
  }while(string[0]!='q');
  free_farray1(mdl,0);
  free_farray1(coef,0);
}

void gridb3d(void){
char disc[4][6],string[40],ifset;
float *coef;
FILE *fp;
int maxs,nknots,ndis,rmax,jump,hmax,mdim,vjump,i,p,nn;
float mindep,maxdep,fac,x,r,*mdl,*knot,*th,thick;

  fp=getfp("r");
  fscanf(fp,"%d %d %d ",&maxs,&nknots,&ndis);
  if(maxs>MAXS) STOP(gridb3d: maxs too large);
  
  if(nknots>=0)
  { mindep=RADIUS-RMOHO;
    maxdep=RADIUS-RCMB;
  }
  else 
  { mindep=0.0;
    maxdep=0.0;
  }
  printf(" Depth range: [%4d %4d] km", (int)mindep,(int)maxdep);
  
  disc[0][0]=disc[1][0]=disc[2][0]=disc[3][0]='\0'; 
  if(ndis)
  { printf(";  Discontinuities:");
    for(i=0;i<ndis;i++)
    { fscanf(fp,"%s",disc[i]);
      printf(" %s",disc[i]);
    }
  }
  printf("\n");
  
  hmax=(maxs+1)*(maxs+1);
  mdim=hmax*(ndis+nknots);
  mdl=farray1(0,mdim-1);
  for(i=0;i<mdim;i++) fscanf(fp,"%g",&mdl[i]);
  fclose(fp);
  
  vjump=hmax*nknots;
  
  coef=farray1(0,hmax-1);
  
  ifset='y';
  
  do
  { printf(" Specify a depth,");
    if(ndis)
    { printf(" or ");
      for(i=0;i<ndis;i++) printf("'%c' ",disc[i][0]);
      printf(" for discontinuity,");
    }
    printf(" or 'q' to quit:");
    scanf("%s",string);
    if(string[0]=='q') break;
    
    if(string[0]=='s') for(i=0;i<hmax;i++) coef[i]=mdl[vjump+i]*6368.0;
    else if(string[0]=='m')
    { if(disc[0][0]=='m') jump=0;
      else if(disc[1][0]=='m') jump=hmax;
      else STOP(grids3d: error in getting moho);
      for(i=0;i<hmax;i++) coef[i]=mdl[vjump+jump+i]*RMOHO;
    }
    else if(string[0]=='d')
    { if(disc[0][0]=='d') jump=0;
      else if(disc[1][0]=='d') jump=hmax;
      else if(disc[2][0]=='d') jump=2*hmax;
      else STOP(gridb3d: error in getting d670);
      for(i=0;i<hmax;i++) coef[i]=mdl[vjump+jump+i]*R670;
    }
    else if(string[0]=='c')
      for(i=0;i<hmax;i++) coef[i]=mdl[vjump+hmax*(ndis-1)+i]*RCMB;
    else if(string[0]>='1' && string[0]<='9'){
      sscanf(string,"%f",&r);
      r=RADIUS-r;
      
      jump=0;
      thick=(RMOHO-RCMB)/(nknots-1);
      knot=farray1(0,nknots-1);
      th=farray1(0,nknots-2);
      for(nn=0;nn<nknots;nn++)knot[nn]=RCMB+thick*(float)nn;
      for(nn=0;nn<nknots-1;nn++)th[nn]=knot[nn+1]-knot[nn];
      
      for(i=0;i<hmax;i++) coef[i]=0.;
      for(p=0;p<nknots;p++){
        fac=100.*cub_bspline(p,nknots,knot,th,r);
        for(i=0;i<hmax;i++) coef[i]+=fac*mdl[jump+i+p*hmax];
      }
    }
    if(string[0]!='q')string[0]='\0';
    harmogrid(maxs,coef,"X^",ifset);
    ifset='n';
  } while(string[0]!='q');

  free_farray1(mdl,0);
  free_farray1(coef,0);
}

void gridA1d(void){
char disc[4][6],string[40],ifset;
float *coef;
FILE *fp;
int maxs,nknots,ndis,rmax,jump,hmax,mdim,vjump,i,p,nn;
float mindep,maxdep,fac,x,r,*mdl,*knot;

  fp=getfp("r");
  fscanf(fp,"%d %d %d ",&maxs,&nknots,&ndis);
  if(maxs>MAXS) STOP(gridA1d: maxs too large);
  
  if(nknots>=0)
  { mindep=RADIUS-RMOHO;
    maxdep=RADIUS-RCMB;
  }
  else 
  { mindep=0.0;
    maxdep=0.0;
  }
  printf(" Depth range: [%4d %4d] km", (int)mindep,(int)maxdep);
  
  disc[0][0]=disc[1][0]=disc[2][0]=disc[3][0]='\0'; 
  if(ndis)
  { printf(";  Discontinuities:");
    for(i=0;i<ndis;i++)
    { fscanf(fp,"%s",disc[i]);
      printf(" %s",disc[i]);
    }
  }
  printf("\n");
  knot=farray1(0,nknots-1);
  for(i=0;i<nknots;i++)fscanf(fp,"%f",&knot[i]);
  
  hmax=(maxs+1)*(maxs+1);
  mdim=hmax*(ndis+nknots);
  mdl=farray1(0,mdim-1);
  for(i=0;i<mdim;i++) fscanf(fp,"%g",&mdl[i]);
  fclose(fp);
  
  vjump=hmax*nknots;
  
  coef=farray1(0,hmax-1);
  
  ifset='y';
  
  do
  { printf(" Specify a depth,");
    if(ndis)
    { printf(" or ");
      for(i=0;i<ndis;i++) printf("'%c' ",disc[i][0]);
      printf(" for discontinuity,");
    }
    printf(" or 'q' to quit:");
    scanf("%s",string);
    if(string[0]=='q') break;
    
    if(string[0]=='s') for(i=0;i<hmax;i++) coef[i]=mdl[vjump+i]*6368.0;
    else if(string[0]=='m')
    { if(disc[0][0]=='m') jump=0;
      else if(disc[1][0]=='m') jump=hmax;
      else STOP(gridA1d: error in getting moho);
      for(i=0;i<hmax;i++) coef[i]=mdl[vjump+jump+i]*RMOHO;
    }
    else if(string[0]=='d')
    { if(disc[0][0]=='d') jump=0;
      else if(disc[1][0]=='d') jump=hmax;
      else if(disc[2][0]=='d') jump=2*hmax;
      else STOP(gridA1d: error in getting d670);
      for(i=0;i<hmax;i++) coef[i]=mdl[vjump+jump+i]*R670;
    }
    else if(string[0]=='c')
      for(i=0;i<hmax;i++) coef[i]=mdl[vjump+hmax*(ndis-1)+i]*RCMB;
    else if(string[0]>='1' && string[0]<='9'){
      sscanf(string,"%f",&r);
      r=RADIUS-r;
      
      jump=0;
            
      for(i=0;i<hmax;i++) coef[i]=0.;
      for(p=0;p<nknots;p++){
        fac=100.*spl(p,nknots,knot,r);
        for(i=0;i<hmax;i++) coef[i]+=fac*mdl[jump+i+p*hmax];
      }
    }
    if(string[0]!='q')string[0]='\0';
    harmogrid(maxs,coef,"X^",ifset);
    ifset='n';
  } while(string[0]!='q');

  free_farray1(mdl,0);
  free_farray1(coef,0);
  free_farray1(knot,0);
}

void gridg3d(void){
char disc[4][6],string[40],ifset;
float *coef;
FILE *fp;
int kmaxw[NPAR],kmaxu[NPAR],kmaxl[NPAR],fcn[NPAR],maxs[NPAR];
int ndis,kmax,hmax,mdim,fldim;
int jump,vjump,mdljump;
int npar,parnum,begmdl,endmdl,begdis;
int np,i,p,kk;
char param[NPAR][2];
float mindep,maxdep,fac,x,r,*mdl,*totmdl;

  fp=getfp("r");
  fscanf(fp,"%d",&npar);  
  if(npar>(NPAR-1))stop("gridg3d: not enough working space (NPAR)");
  for(np=0;np<npar;np++){
    fscanf(fp,"%d %d %d %d %d %s",&maxs[np],&kmaxw[np],&kmaxu[np],&kmaxl[np],&fcn[np],param[np]);
    if(maxs[np]>MAXS) stop("gridg3d: maxs too large");
  }
  fscanf(fp,"%d",&ndis);
  printf("%d parameters model:\n",npar);
  for(np=0;np<npar;np++)printf("%d: %s\n",np,param[np]);
  printf("Select parameter # > ");
  scanf("%d",&parnum);
  if(parnum>npar-1)stop("Invalid parameter number");
  printf("Parameter selected: %s\n",param[parnum]);
  
  if(kmaxw[parnum]>=0 || (kmaxu[parnum]>=0 && kmaxl[parnum]>=0) ){
      mindep=RADIUS-RMOHO;
      maxdep=RADIUS-RCMB;
  }
  else if(kmaxu[parnum]>=0){
    mindep=RADIUS-RMOHO;
    maxdep=RADIUS-R670;
  }
  else if(kmaxl[parnum]>=0){
    mindep=RADIUS-R670;
    maxdep=RADIUS-RCMB;
  }
  else {
    mindep=0.0;
    maxdep=0.0;
  }
  printf(" Depth range: [%4d %4d] km", (int)mindep,(int)maxdep);
  for(i=0;i<4;i++)disc[i][0]='\0';
  if(ndis){
    printf(";  Discontinuities:");
    for(i=0;i<ndis;i++){
      fscanf(fp,"%s",disc[i]);
      printf(" %s",disc[i]);
    }
  }
  printf(" or 'q' to quit:");
  printf("\n");
  
  hmax=(maxs[parnum]+1)*(maxs[parnum]+1);
  mdim=hmax*(ndis+kmaxw[parnum]+1+kmaxu[parnum]+1+kmaxl[parnum]+1);
  for(fldim=0,np=0;np<npar;np++)
    fldim+=(maxs[np]+1)*(maxs[np]+1)*(kmaxw[np]+kmaxu[np]+kmaxl[np]+3);
  begdis=fldim;
  fldim+=hmax*ndis;
  
  if(parnum==0)
    begmdl=0;
  else
    for(begmdl=0,np=1;np<npar;np++)
      begmdl+=(maxs[np-1]+1)*(maxs[np-1]+1)*(kmaxw[np-1]+kmaxu[np-1]+kmaxl[np-1]+3);
  endmdl=begmdl+(kmaxu[parnum]+kmaxl[parnum]+kmaxw[parnum]+3)*(maxs[parnum]+1)*(maxs[parnum]+1)-1;
  /* Load entire model */
  totmdl=farray1(0,fldim-1);
  for(i=0;i<fldim;i++) fscanf(fp,"%g",&totmdl[i]);
  fclose(fp);
  
  mdl=farray1(0,mdim-1);
  for(i=0,kk=0;kk<fldim;kk++){
    if((kk>=begmdl&&kk<=endmdl)||(kk>=begdis)){
      mdl[i]=totmdl[kk];
      i++;
    }
  }
  free_farray1(totmdl,0);
  
  vjump=hmax*(kmaxw[parnum]+kmaxu[parnum]+kmaxl[parnum]+3);
  
  coef=farray1(0,hmax-1);
  
  ifset='y';
  do {
    printf(" Specify a depth,");
    if(ndis){
      printf(" or ");
      for(i=0;i<ndis;i++) printf("'%c' ",disc[i][0]);
      printf(" for discontinuity,");
    }
    scanf("%s",string);
    if(string[0]=='q')break;
    
    if(string[0]=='s') for(i=0;i<hmax;i++) coef[i]=mdl[vjump+i]*6368.0;
    else if(string[0]=='m')
    { if(disc[0][0]=='m') jump=0;
      else if(disc[1][0]=='m') jump=hmax;
      else STOP(grids3d: error in getting moho);
      for(i=0;i<hmax;i++) coef[i]=mdl[vjump+jump+i]*RMOHO;
    }
    else if(string[0]=='d')
    { if(disc[0][0]=='d') jump=0;
      else if(disc[1][0]=='d') jump=hmax;
      else if(disc[2][0]=='d') jump=2*hmax;
      else STOP(grids3d: error in getting d670);
      for(i=0;i<hmax;i++) coef[i]=mdl[vjump+jump+i]*R670;
    }
    else if(string[0]=='c')
      for(i=0;i<hmax;i++) coef[i]=mdl[vjump+hmax*(ndis-1)+i]*RCMB;
    else if(string[0]>='1' && string[0]<='9')
    { sscanf(string,"%f",&r);
      r=RADIUS-r;
      
      if(kmaxw[parnum]<0)
      { if(r>R670)
        { if(kmaxu[parnum]<0) STOP(grids3d: upper mantle not available);
	  x=-1.+2.*(r-R670)/(RMOHO-R670);
	  kmax=kmaxu[parnum];
	  jump=0;
        }
        else
        { if(kmaxl[parnum]<0) STOP(grids3d: lower mantle not available);
	  x=-1.+2.*(r-RCMB)/(R670-RCMB);
	  kmax=kmaxl[parnum];
	  jump=(kmaxu[parnum]+1)*hmax;
        }
      }
      else
      { if(kmaxu[parnum]>=0 || kmaxl[parnum]>=0) STOP(grids3d: error in s3d file);
	x=-1.+2.*(r-RCMB)/(RMOHO-RCMB);
	kmax=kmaxw[parnum];
	jump=0;
      }
      
      for(i=0;i<hmax;i++) coef[i]=0;
      for(p=0;p<=kmax;p++)
      { fac=100.*Legpolyn(p,x);
        for(i=0;i<hmax;i++) coef[i]+=fac*mdl[jump+i+p*hmax];
      }
    }
    if(string[0]!='q')string[0]='\0';
    harmogrid(maxs[parnum],coef,"X^",ifset);

    ifset='n';
  }while(string[0]!='q');
  free_farray1(mdl,0);
  free_farray1(coef,0);
}

void harms3d(void){
char disc[4][6],string[40],ifset;
float *coef;
FILE *fp,*outf;
int maxs,kmaxw,kmaxu,kmaxl,ndis,kmax,jump,hmax,mdim,vjump,i,p;
float mindep,maxdep,fac,x,r,*mdl;

  fp=getfp("r");
  fscanf(fp,"%d %d %d %d %d",&maxs,&kmaxw,&kmaxu,&kmaxl,&ndis);
  if(maxs>MAXS) STOP(harms3d: maxs too large);
  
  if(kmaxw>=0 || (kmaxu>=0 && kmaxl>=0) )
  { mindep=RADIUS-RMOHO;
    maxdep=RADIUS-RCMB;
  }
  else if(kmaxu>=0)
  { mindep=RADIUS-RMOHO;
    maxdep=RADIUS-R670;
  }
  else if(kmaxl>=0)
  { mindep=RADIUS-R670;
    maxdep=RADIUS-RCMB;
  }
  else 
  { mindep=0.0;
    maxdep=0.0;
  }
  printf(" Depth range: [%4d %4d] km", (int)mindep,(int)maxdep);
  
  disc[0][0]=disc[1][0]=disc[2][0]=disc[3][0]='\0'; 
  if(ndis)
  { printf(";  Discontinuities:");
    for(i=0;i<ndis;i++)
    { fscanf(fp,"%s",disc[i]);
      printf(" %s",disc[i]);
    }
  }
  printf(" or 'q' to quit:");
  printf("\n");
  
  hmax=(maxs+1)*(maxs+1);
  mdim=hmax*(ndis+kmaxw+1+kmaxu+1+kmaxl+1);
  mdl=farray1(0,mdim-1);
  for(i=0;i<mdim;i++) fscanf(fp,"%g",&mdl[i]);
  fclose(fp);
  
  vjump=hmax*(kmaxw+1+kmaxu+1+kmaxl+1);
  
  coef=farray1(0,hmax-1);
  
  ifset='y';
  do {
    printf(" Specify a depth,");
    if(ndis)
    { printf(" or ");
      for(i=0;i<ndis;i++) printf("'%c' ",disc[i][0]);
      printf(" for discontinuity,");
    }
    scanf("%s",string);
    if(string[0]=='q')break;
    
    if(string[0]=='s') for(i=0;i<hmax;i++) coef[i]=mdl[vjump+i]*6368.0;
    else if(string[0]=='m')
    { if(disc[0][0]=='m') jump=0;
      else if(disc[1][0]=='m') jump=hmax;
      else STOP(grids3d: error in getting moho);
      for(i=0;i<hmax;i++) coef[i]=mdl[vjump+jump+i]*RMOHO;
    }
    else if(string[0]=='d')
    { if(disc[0][0]=='d') jump=0;
      else if(disc[1][0]=='d') jump=hmax;
      else if(disc[2][0]=='d') jump=2*hmax;
      else STOP(grids3d: error in getting d670);
      for(i=0;i<hmax;i++) coef[i]=mdl[vjump+jump+i]*R670;
    }
    else if(string[0]=='c')
      for(i=0;i<hmax;i++) coef[i]=mdl[vjump+hmax*(ndis-1)+i]*RCMB;
    else if(string[0]>='1' && string[0]<='9')
    { sscanf(string,"%f",&r);
      r=RADIUS-r;
      
      if(kmaxw<0)
      { if(r>R670)
        { if(kmaxu<0) STOP(grids3d: upper mantle not available);
	  x=-1.+2.*(r-R670)/(RMOHO-R670);
	  kmax=kmaxu;
	  jump=0;
        }
        else
        { if(kmaxl<0) STOP(grids3d: lower mantle not available);
	  x=-1.+2.*(r-RCMB)/(R670-RCMB);
	  kmax=kmaxl;
	  jump=(kmaxu+1)*hmax;
        }
      }
      else
      { if(kmaxu>=0 || kmaxl>=0) STOP(grids3d: error in s3d file);
	x=-1.+2.*(r-RCMB)/(RMOHO-RCMB);
	kmax=kmaxw;
	jump=0;
      }
      
      for(i=0;i<hmax;i++) coef[i]=0;
      for(p=0;p<=kmax;p++)
      { fac=Legpolyn(p,x);
        for(i=0;i<hmax;i++) coef[i]+=fac*mdl[jump+i+p*hmax];
      }
    }
    if(string[0]!='q')string[0]='\0';
    /*Assumes depth is selected: modify to include discontinuity */
    depthharm(maxs,coef,ifset,(float)RADIUS-r);
    
    ifset='n';
  }while(string[0]!='q');
  free_farray1(mdl,0);
  free_farray1(coef,0);
}
