/* generates a grid for given spherical harmonic coefficients 
   grid output to file  
*/

#include <stdio.h>
#include <math.h>
#include "LIB/glob.h"
#include "LIB/constmap.h"

#define NAME 50

extern float **farray2();
extern void free_farray2();
extern float *assolegendre(float theta,int l,char *conventn);

FILE *gridf;

void harmogrid(int maxs,float *coef,char *conven,char ifset){
/* spherical harmonic coefficients: a, b
 * f=sum (a*cos(m*phi)+b*sin(m*phi))*P(theta)
 */
/*convention of spherical harmonic: P, p, X, X^, P^*/
/* For s3d files, convention must be X^ */
/*'y': set smin,smax,aeo*/
float **grid,**cossintphi,**p,*y,fac;
static int smin,smax;
static char aeo[2];
float theta,phi,sum;
int s,t,index,i,j,k,imax;
float cosphi,sinphi,costph,sintph,temp;
char name[NAME];

        if(ifset=='y')
        { printf("\nMax degree of model is %d",maxs);
	  do
	  { printf("\nsmin & smax of grid :");
	    scanf("%d %d",&smin,&smax);
	  } while(smax>maxs);
	
	  do
	  { printf("\nType 'a' to grid  all degrees");
	    printf("\n     'e' to grid even degrees only");
	    printf("\n or  'o' to grid  odd degrees only  :");
	    scanf("%s",aeo);
	  } while(aeo[0]!='a' && aeo[0]!='e' && aeo[0]!='o');
	}
        maxs=smax;
        
        imax=(maxs+1)*(maxs+1);
	
	for(index=0,s=0;s<smin;s++)
	for(t=0;t<2*s+1;t++)
	  coef[index++]=0.0;
	
	if(aeo[0]!='a')
	for(s=smin;s<=smax;s++)
	{ fac=1.;
	  if( (s%2 && aeo[0]=='e') || (!(s%2) && aeo[0]=='o') ) fac=0.;
	  for(t=0;t<2*s+1;t++)
	    coef[index++]*=fac;
	}

	p=farray2(0,Row-1,0,imax-1);
	for(i=0;i<Row;i++)
	{ theta=CONV*Dy*(float)i;
	  if(theta<0.0) theta=0.0;
	  if(theta>PI) theta=PI;
	  for(index=-1,s=0;s<=maxs;s++)
	  { y=assolegendre(theta,s,conven);
	    for(t=0;t<=s;t++)
	    { p[i][++index]=y[t];
	      if(t) p[i][++index]=y[t];
	    }
	  }
	}
	
	cossintphi=farray2(0,Col-1,0,imax-1);
	for(j=0;j<Col;j++)
	{ phi=CONV*((float)j*Dx);
	  cosphi=(float)cos((double)phi);
	  sinphi=(float)sin((double)phi);
	  for(index=-1,s=0;s<=maxs;s++)
	  for(costph=1.0,sintph=0.0,t=0;t<=s;t++)
	  { cossintphi[j][++index]=costph;
	    if(t) cossintphi[j][++index]=sintph;
	    temp=costph*cosphi-sintph*sinphi;
	    sintph=sintph*cosphi+costph*sinphi;
	    costph=temp;
	  }
	}
	
	grid=farray2(0,Row-1,0,Col-1);
	for(i=0;i<Row;i++)
	for(j=0;j<Col;j++)
	{ for(sum=0.0,k=0;k<imax;k++)
	    sum+=coef[k]*cossintphi[j][k]*p[i][k];
	  grid[i][j]=sum;
	}
	free_farray2(p,0,Row-1,0);
	
	printf("Enter name of output file > ");
	scanf("%s",name);
	if((gridf=fopen(name,"w"))==0)stop("Cannot open output file");
	for(i=0;i<Row;i++) {
          theta=Dy*(float)i;
	  for(j=0;j<Col;j++) {
	    phi=((float)j*Dx);
	    if(phi!=360.)
	      fprintf(gridf,"%.1f\t%.1f\t%.3f\n",phi,90-theta,grid[i][j]);
	  }
	}
	fclose(gridf);
}

void depthharm(maxs,coef,ifset,depth)
int maxs;
float *coef, depth;
char ifset;   /*'y': set smin,smax,aeo*/
{
float **grid,**cossintphi,**p,*y,fac;
static int smin,smax;
static char aeo[2];
float theta,phi,sum;
int s,t,index,i,j,k,imax,hh;
float cosphi,sinphi,costph,sintph,temp;
char name[NAME];

        if(ifset=='y')
        { printf("\nMax degree of model is %d",maxs);
	  do
	  { printf("\nsmin & smax of grid :");
	    scanf("%d %d%*c",&smin,&smax);
	  } while(smax>maxs);
	
	  do
	  { printf("\nType 'a' to grid  all degrees");
	    printf("\n     'e' to grid even degrees only");
	    printf("\n or  'o' to grid  odd degrees only  :");
	    scanf("%s",aeo);
	  } while(aeo[0]!='a' && aeo[0]!='e' && aeo[0]!='o');
	}
        maxs=smax;
        
        imax=(maxs+1)*(maxs+1);
	
	for(index=0,s=0;s<smin;s++)
	for(t=0;t<2*s+1;t++)
	  coef[index++]=0.0;
	
	if(aeo[0]!='a')
	for(s=smin;s<=smax;s++)
	{ fac=1.;
	  if( (s%2 && aeo[0]=='e') || (!(s%2) && aeo[0]=='o') ) fac=0.;
	  for(t=0;t<2*s+1;t++)
	    coef[index++]*=fac;
	}
	
	printf("Enter name of output file > ");
	scanf("%s",name);
	if((gridf=fopen(name,"w"))==0)stop("Cannot open output file");
	fprintf(gridf,"%.1f\n",depth);
	for(hh=0,i=0;i<=maxs;i++) {
	  for(j=0;j<2*i+1;j++) {
	    fprintf(gridf,"%9.4g ",coef[hh++]);
	  }
	  fprintf(gridf,"\n");
	}
	if(hh!=imax){
	  printf("hh=%d imax=%d\n",hh,imax);
	  stop("depthharm: dimension error");
	}
	fclose(gridf);
}
